(***********************************************************************)
(*                                                                     *)
(*                                 Pio                                 *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(*  $Id: gen_test_file.ml,v 1.3 2011-05-03 10:35:01 fclement Exp $ *)

open Write_typed_value;;
open Write_communication;;

type output_mode =
   | Output_communication
   | Output_typed_value
;;

let get_output_mode, set_output_mode = Uaref.make_ref Output_communication;;

let set_communication () = set_output_mode Output_communication;;
let set_typed_value () = set_output_mode Output_typed_value;;

Ucommand_option.add_all [
  {
   Ucommand_option.key = "--communication";
   spec = Arg.Unit set_communication;
   doc =
     "  Turn on the generation of a communication r/w test file.\
     \n\tThis is the default.\n";
  };

  {
   Ucommand_option.key = "--typed_value";
   spec = Arg.Unit set_typed_value;
   doc =
     "  Turn on the generation of a typed_value r/w test file.\
     \n\tThis is the default.\n";
  };
]
;;

let usage_msg = Printf.sprintf "\nUsage: %s [options]\n" Sys.argv.(0);;

let parse_cmd_line () =
  Ucommand_option.parse
    (fun _ -> prerr_endline usage_msg; exit 2)
    usage_msg
;;

(* Check general write communications on a buffer *)

let write_and_flush write ob t =
  write ob t;
  Ubuffer.flush_buffer_on_stdout ob;
  if Debug_io.get_debug () then Unix.sleep 1
;;

let main () =
  parse_cmd_line ();
  let ob = Buffer.create 1024 in
  if get_output_mode () = Output_typed_value
  then List.iter (write_and_flush write_typed_value ob) Gen_typed_values.list
  else List.iter (write_and_flush write_communication ob) Gen_communication.list
;;

main ()
;;

exit 0
;;
