(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2007-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parser.ml,v 1.5.2.1 2011-09-21 13:08:04 fclement Exp $ *)

type 'a parser = { parse : string -> ('a * string) option; };;

let parse p s =
  match p.parse s with
  | Some (x, _) -> Some x
  | None -> None
;;

let full_parse p = p.parse;;

let return x = { parse = fun s -> Some (x, s) };;
let zero = { parse = fun _ -> None };;
let bind p f =
  { parse = fun s ->
    match p.parse s with
    | Some (x, s') ->
      (f x).parse s'
    | None -> None }
;;

let ( >>= ) = bind;;
let ( >> ) p0 p1 = p0 >>= (fun _ -> p1);;

let plus p0 p1 =
  { parse = fun s ->
    let ret = p0.parse s in
    if ret <> None then ret else p1.parse s }
;;

let ( +++ ) = plus;;

let char c =
  { parse = fun s ->
    if String.length s = 0 then None else
    if s.[0] <> c then None else
    Some (c, String.sub s 1 (String.length s - 1)) }
;;

let eof =
  { parse = fun s ->
    if String.length s = 0 then Some ((), "") else None }
;;

let rec any = function
  | p :: ps -> p +++ (any ps)
  | [] -> zero
;;

let many p =
  let rec many acc =
  plus
    (p >>= (fun c ->
      many (c :: acc)))
    (return (List.rev acc)) in
  many []
;;

let many1 p =
  p >>= (fun x -> many p >>= (fun l -> return (x :: l)))
;;

let many_till pcond p =
  let rec many_till acc =
    (pcond >> return (List.rev acc)) +++
    (p >>= fun x -> many_till (x :: acc)) in
  many_till []
;;

let maybe p =
  plus (p >>= (fun x -> return (Some x))) (return None)
;;

let times n p =
  let rec times n acc =
    if n = 0 then return (List.rev acc) else
    p >>= (fun x -> times (n - 1) (x :: acc)) in
  times n []
;;

let less_than n p =
  let rec less_than n acc =
    if n = 0 then return (List.rev acc) else
    (p >>= (fun x -> less_than (n - 1) (x :: acc))) +++
    return (List.rev acc) in
  less_than n []
;;
